#include "lithe_barrier.h"
#include "spmd.h"
#include "atomic.h"
#include <assert.h>
#include <stdlib.h>
#include <sys/sysinfo.h>


typedef struct {
  int nthreads;
  lithe_barrier_t barrier;
  int sum;
} data_t;


void lockstep(void *__dptr) 
{
  data_t *dptr = (data_t *)__dptr;
  assert(dptr != NULL);
  atomic_add(&dptr->sum, spmd_tid());
  lithe_barrier_wait(&dptr->barrier);
  assert(dptr->sum == (((dptr->nthreads - 1) * dptr->nthreads) / 2));
  lithe_barrier_wait(&dptr->barrier);
  atomic_add(&dptr->sum, spmd_tid());
  lithe_barrier_wait(&dptr->barrier);
  assert(dptr->sum == ((dptr->nthreads - 1) * dptr->nthreads));  
}


int main(int argc, char **argv) 
{
  data_t data;
  for (data.nthreads = 1; data.nthreads < get_nprocs() * 3; data.nthreads += 1) {
    lithe_barrier_init(&data.barrier, data.nthreads);
    data.sum = 0;
    spmd_spawn(data.nthreads, lockstep, (void *)&data);
    lithe_barrier_destroy(&data.barrier);
  }
  return 0;
}
