#ifndef SPMD_INTERNAL_H
#define SPMD_INTERNAL_H


#include "taskqueue.h"
#include "lithe.hh"
#include "spinlock.h"
#include <assert.h>


class SPMD_Scheduler : public Lithe::Scheduler 
{
 public:
  SPMD_Scheduler(int __nthreads, void (*__func)(void *), void *__arg);
  ~SPMD_Scheduler();
 protected:
  void spawn();
  void enter_begin();
 protected:
  void enter();
  void yield(lithe_sched_t *child) { assert(0); };
  void reg(lithe_sched_t *child) { assert(0); };
  void unreg(lithe_sched_t *child) { assert(0); };
  void request(lithe_sched_t *child, int k) { assert(0); };
  void unblock(lithe_task_t *task);
 protected:
  int nthreads;
 private:
  void (*func)(void *);
  void *arg;
  int spawned;
  int completed;
  int unreg_once;
  TaskQueue *unblocked;
 private:
  friend void spmd_spawn(int nthreads, void (*func)(void *), void *arg);
  friend int spmd_tid();
  friend void __spawn(void *__sched);
  friend void __enter_begin(void *__sched);
};


enum { ACTION_YIELD = 0, ACTION_UNREG = 1 };


void __spawn(void *__sched);
void __enter_begin(void *__sched);
void __enter_end(lithe_task_t *task, void *__action);


#endif  // SPMD_INTERNAL_H
