#ifndef TASKQUEUE_H
#define TASKQUEUE_H


#include "lithe.h"
#include "spinlock.h"
#include <stdlib.h>
#include <assert.h>


class TaskQueue
{
 public:
  TaskQueue(size_t __max);
  ~TaskQueue();
 public:
  void push(lithe_task_t *task);
  lithe_task_t * pop();
 private:
  inline bool empty();
  inline bool full();
 private:
  lithe_task_t **queue;
  size_t head;
  size_t tail;
  size_t max;
  int mtx;
};


TaskQueue::TaskQueue(size_t __max) :
  head(0),
  tail(0),
  max(__max)
{
  queue = (lithe_task_t **)malloc((max + 1) * sizeof(lithe_task_t *));
  assert(queue != NULL);
  spinlock_init(&mtx);
}


TaskQueue::~TaskQueue() 
{
  free(queue);
}


void TaskQueue::push(lithe_task_t *task) 
{
  spinlock_lock(&mtx);
  assert(!full());
  queue[tail] = task;
  tail = ((tail == max) ? 0 : (tail + 1));
  assert(!empty());
  spinlock_unlock(&mtx);
}


lithe_task_t * TaskQueue::pop() 
{
  lithe_task_t *task = NULL;
  spinlock_lock(&mtx);
  if (!empty()) {
    task = queue[head];
    head = ((head == max) ? 0 : (head + 1));
  }
  spinlock_unlock(&mtx);
  return task;
}


bool TaskQueue::empty() 
{
  return (head == tail);
}


bool TaskQueue::full() 
{
  if (tail >= head) {
    return ((head == 0) && (tail == max));
  } else {
    return ((head - tail) == 1);
  }
}


#endif  // TASKQUEUE_H
